from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.error_model import ErrorModel


T = TypeVar("T", bound="ErrorResponse")


@_attrs_define
class ErrorResponse:
    """Define error response model.

    Attributes:
        error (ErrorModel): ErrorModel class object instance.

    Raises:
        pydantic.error_wrappers.ValidationError: If any of provided attribute
            doesn't pass type validation.

        Attributes:
            error (ErrorModel): Define base error model for the response.

                Attributes:
                    code (int): HTTP error status code.
                    message (str): Detail on HTTP error.
                    status (str): HTTP error reason-phrase as per in RFC7235. NOTE! Set
                        automatically based on HTTP error status code.

                Raises:
                    pydantic.error_wrappers.ValidationError: If any of provided attribute
                        doesn't pass type validation.
    """

    error: "ErrorModel"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        error = self.error.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "error": error,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.error_model import ErrorModel

        d = src_dict.copy()
        error = ErrorModel.from_dict(d.pop("error"))

        error_response = cls(
            error=error,
        )

        error_response.additional_properties = d
        return error_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
