from pydantic import BaseModel, model_validator

# Generic Models

## Node Model

## Relationship Model

# Objects

## User model

## Site model

## Organization model

# Relationship model

## Groups model

# Will have the ability to add a user to a group. We can set application (organization with application).

## Global Group Model

# Inherit from the group model. This will be a group that is global to the application.


## Organization User Model

## Organization Site Model
